--[[	PhantomPlates Classification Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

------------------
--[[	Hooks	]]
------------------
local TextureObjects={};

local function UpdateClassification(self,unit)
	if not UnitIsPlayer(unit) then
		local classification=UnitClassification(unit);
		local showrare=AddOn.Options.Classification_ShowRare
			and (
				classification=="rare"
				or classification=="rareelite"
			);
		local showelite=AddOn.Options.Classification_ShowElite
			and (
				classification=="worldboss"
				or classification=="elite"
				or classification=="rareelite"
			);

		self.Border:SetDesaturated(showrare);
		self.Classification:SetShown(showelite);
		if showelite then
			self.Classification:SetTexture(showrare
				and "Interface\\Tooltips\\RareEliteNameplateIcon"
				or "Interface\\Tooltips\\EliteNameplateIcon"
			);
		end
	else self.Border:SetDesaturated(false); self.Classification:Hide(); end
end

AddOn.OnNamePlateCreated:AddCallback(function(_,base)
	local unitframe=base.UnitFrame;
	if not (unitframe:IsForbidden() or TextureObjects[unitframe]) then
		local parent=unitframe.healthBar.border;

		local border; do--	Border is unnamed and not stored in its parent
			for _,region in ipairs({parent:GetRegions()}) do
				if region.GetTexture and region:GetTexture()=="Interface/Tooltips/Nameplate-Border" then
					border=region; break;
				end
			end
		end

		local classification=parent:CreateTexture(nil,"ARTWORK");
		classification:SetDrawLayer("ARTWORK",1);
		classification:SetSize(64,32);
		classification:SetPoint("LEFT",parent,"RIGHT",-26,-3);
		classification:Hide();

		TextureObjects[unitframe]={Border=border,Classification=classification};
	end
end);

AddOn.OnNamePlateAdded:AddCallback(function(base,unit)
	local objects=TextureObjects[base.UnitFrame];
	if objects then return UpdateClassification(objects,unit); end
end);

--------------------------
--[[	Options Setup	]]
--------------------------
local OptionsLayoutSection=1;--AddOn.GetPreferredOptionsLayoutSection();
AddOn.AddOptionsLayoutPadding(OptionsLayoutSection);

local function OnOptionsUpdate(_,key,val)
	for unitframe,objects in pairs(TextureObjects) do
		local unit=unitframe.displayedUnit;
		if unit then UpdateClassification(objects,unit); end
	end
end

AddOn.RegisterOptionsControl("Classification_ShowRare",true
	,"OnClick","SetChecked","GetChecked"
	,OnOptionsUpdate
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24));
		button.text:SetText(AddOn.Localization.OptionsSetting_Classification_ShowRare);
		return button;
	end
);

AddOn.RegisterOptionsControl("Classification_ShowElite",true
	,"OnClick","SetChecked","GetChecked"
	,OnOptionsUpdate
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24));
		button.text:SetText(AddOn.Localization.OptionsSetting_Classification_ShowElite);
		return button;
	end
);
