--[[	PhantomPlates NamePlate Text Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

----------------------------------
--[[	Import From Utilities	]]
----------------------------------
local AddOn_Utilities_UnitPowerBarColor=AddOn.Utilities.UnitPowerBarColor;
local AddOn_Utilities_AdjustColorBrightness=AddOn.Utilities.AdjustColorBrightness;
local AddOn_Utilities_ShouldKnowUnitHealth=AddOn.Utilities.ShouldKnowUnitHealth;

----------------------------------
--[[	Local References	]]
----------------------------------
local math_abs=math.abs;
local math_floor=math.floor;
local math_log10=math.log10;
local math_max=math.max;

----------------------------------
--[[	Number Formatting	]]
----------------------------------
local NumberCaps={FIRST_NUMBER_CAP,SECOND_NUMBER_CAP};
local AddOn_Localization_NumberGroupingScale=AddOn.Localization.NumberGroupingScale;
local function AbbreviateNumber(val)--	Abbreviates large numbers
	local exp=math_max(0,math_floor(math_log10(math_abs(val))));--	Calculate exponent of 10 and clamp to zero
	if exp<AddOn_Localization_NumberGroupingScale then return tostring(math_floor(val)); end--	Less than 1k, return as-is

	local factor=math_floor(exp/AddOn_Localization_NumberGroupingScale);--	Exponent factor of 1k
	local precision=math_max(0,(AddOn_Localization_NumberGroupingScale-1)-exp%AddOn_Localization_NumberGroupingScale);--	Dynamic precision based on how many digits we have (Returns numbers like 100k, 10.0k, and 1.00k)

--	Fallback to scientific notation if we run out of units
	return (("%s%%0.%df%%s"):format(val<0 and "-" or "",precision)):format(val/(10^AddOn_Localization_NumberGroupingScale)^factor,NumberCaps[factor] or "e"..(factor*AddOn_Localization_NumberGroupingScale));
end

------------------
--[[	Hooks	]]
------------------
local TextObjects={};
local DEAD=("|cff606060%s|r"):format(DEAD);
local GHOST=("|cff3030c0%s|r"):format(DEAD);
local UNCONSCIOUS=("|cffff00ff%s|r"):format(UNCONSCIOUS);

--	CastBar Hook (Init to a built-in NoOp function for Classic Vanilla)
local NamePlateCastBar_OnUpdate=nop; if not AddOn.IsVanillaClient then--	function NamePlateCastBar_OnUpdate(self,elapsed)
	local DiffHoldTimer,DiffFadeTimer=0.5,0.5;
	local DiffFullTimer=DiffHoldTimer+DiffFadeTimer;

	local CastTracking={};
	function NamePlateCastBar_OnUpdate(self,elapsed)
		if self:IsForbidden() then return; end
		local parent=self:GetParent();
		if not TextObjects[parent] then return; end

		local CastTimeEnabled=AddOn.Options.NamePlateText_ShowCastTime;
		local CastDiffEnabled=AddOn.Options.NamePlateText_ShowCastDiff;
		if not (CastTimeEnabled or CastDiffEnabled) then return; end

		local timetext,difftext=unpack(TextObjects[parent],3)

		local track=CastTracking[self];
		if not track then track={}; CastTracking[self]=track; end
		if track.DiffTimer then--	Tick fade timer
			if track.DiffTimer<=elapsed then track.DiffTimer=nil;
			else track.DiffTimer=track.DiffTimer-elapsed; end
		end

		local diff=0;
		if self.casting or self.channeling then
			local cur,max=self.value,self.maxValue;
			local castid=(self.casting and self.castID or select(7,UnitChannelInfo(self.unit)));--	Channeling doesn't have a CastID, using SpellID instead

			if castid==track.CastID then
				diff,track.Value=cur-track.Value+(self.casting and -elapsed or elapsed),cur;
				if diff<0 then track.DiffTimer=DiffFullTimer; end--	Reset fade timer to full
			else track.CastID,track.Value=castid,cur; end

			if CastTimeEnabled then timetext:SetFormattedText("%0.1f / %0.1fs",cur,max); end
			if CastDiffEnabled and diff<0 then difftext:SetFormattedText("%0.1fs",diff); end
		else track.CastID=nil; end--	Not casting

		if CastDiffEnabled then difftext:SetAlpha(math.min((track.DiffTimer or 0)/DiffFadeTimer,1)); end
	end
	hooksecurefunc("CastingBarFrame_OnUpdate",NamePlateCastBar_OnUpdate);
end

AddOn.OnNamePlateCreated:AddCallback(function(_,base)
	local unitframe=base.UnitFrame;
	if not (unitframe:IsForbidden() or TextObjects[unitframe]) then
		local health=unitframe.healthBar:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
		AddOn.Utilities.ModifyFont(health,nil,nil,"OUTLINE");
		health:SetJustifyH("LEFT");
		health:SetPoint("LEFT");
		health:SetTextColor(0,1,0);

		local power=unitframe.healthBar:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
		AddOn.Utilities.ModifyFont(power,nil,nil,"OUTLINE");
		power:SetJustifyH("RIGHT");
		power:SetPoint("RIGHT");
		power:SetTextColor(ColorMixin.GetRGB(PowerBarColor.MANA));

--		Already created by Blizzard, just needs an anchor
		unitframe.statusText:SetParent(unitframe.healthBar);
		unitframe.statusText:SetFontObject("SystemFont_Shadow_Med1_Outline");
		unitframe.statusText:SetPoint("CENTER",unitframe,"CENTER",0,-6);

		local castremain,castdiff; if not AddOn.IsVanillaClient then
			castremain=unitframe.CastBar:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
			AddOn.Utilities.ModifyFont(castremain,nil,nil,"OUTLINE");
			castremain:SetJustifyH("LEFT");
			castremain:SetPoint("LEFT",0,2);

			castdiff=unitframe.CastBar:CreateFontString(nil,"OVERLAY","NumberFontNormalSmall");
			AddOn.Utilities.ModifyFont(castdiff,nil,nil,"OUTLINE");
			castdiff:SetJustifyH("RIGHT");
			castdiff:SetPoint("RIGHT",0,2);
			castdiff:SetTextColor(1,0,0);

--			Backup for hooking CastingBarFrame_OnUpdate() (Keeping this here in case that fails in the future)
--			unitframe.CastBar:HookScript("OnUpdate",NamePlateCastBar_OnUpdate);
		end

		if not AddOn.Options.NamePlateText_ShowHealth then health:Hide(); end
		if not AddOn.Options.NamePlateText_ShowPower then power:Hide(); end
		if not AddOn.IsVanillaClient then
			if not AddOn.Options.NamePlateText_ShowCastTime then castremain:Hide(); end
			if not AddOn.Options.NamePlateText_ShowCastDiff then castdiff:Hide(); end
		end
		TextObjects[unitframe]={health,power,castremain,castdiff};
	end
end);

--	These functions are shared with other UnitFrames
local function UpdateAnchors(self)
	if not TextObjects[self] then return; end
	local health,power=unpack(TextObjects[self]);

	local showhealth,showpower=health:IsShown(),power:IsShown();
	if showhealth and showpower then
		local unit=self.displayedUnit;

		health:ClearAllPoints(); power:ClearAllPoints();
		if UnitIsUnconscious(unit) or UnitIsDeadOrGhost(unit) then health:SetPoint("CENTER");
		else health:SetPoint("BOTTOMLEFT",self.healthBar,"LEFT"); end
		power:SetPoint("TOPRIGHT",self.healthBar,"RIGHT",2,0);
	elseif showhealth then health:ClearAllPoints(); health:SetPoint("LEFT");
	elseif showpower then power:ClearAllPoints(); power:SetPoint("RIGHT");
	end
end

local function UpdateHealth(self)
	if not (TextObjects[self] and AddOn.Options.NamePlateText_ShowHealth) then return; end
	local text,unit=TextObjects[self][1],self.displayedUnit;

	if not UnitIsConnected(unit) or UnitIsDeadOrGhost(unit) then
		text:Hide(); return;
	else text:Show(); end

	local cur=UnitHealth(self.displayedUnit);
	text:SetText(AddOn_Utilities_ShouldKnowUnitHealth(unit)
	and	(AddOn.Options.NamePlateText_AbbreviateNumber and AbbreviateNumber(cur) or cur)
	or	cur.."%"
	);
end

local function UpdatePower(self)
	if not (TextObjects[self] and AddOn.Options.NamePlateText_ShowPower) then return; end
	local text,unit=TextObjects[self][2],self.displayedUnit;
	if UnitIsDeadOrGhost(unit) then text:Hide(); else
		local powertype=UnitPowerType(unit);
		local cur,max=UnitPower(unit,powertype),UnitPowerMax(unit,powertype);
		if max>0 and cur>0 then
			text:SetText(AddOn.Options.NamePlateText_AbbreviateNumber and AbbreviateNumber(cur) or cur);
			text:Show();
		else text:Hide(); end
	end
end

local function UpdatePowerColor(self)
	if not (TextObjects[self] and AddOn.Options.NamePlateText_ShowPower) then return; end
	TextObjects[self][2]:SetTextColor(AddOn_Utilities_AdjustColorBrightness(0.375,AddOn_Utilities_UnitPowerBarColor(self.displayedUnit)));
end

--	:OnAdded() calls CompactUnitFrame_UpdateAll(), which calls these anyway
--	Fire UpdateAnchors() reads :IsShown() state set by Update*()
hooksecurefunc("CompactUnitFrame_UpdateHealth",UpdateHealth);
hooksecurefunc("CompactUnitFrame_UpdateHealth",UpdateAnchors);
hooksecurefunc("CompactUnitFrame_UpdateMaxPower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdateMaxPower",UpdateAnchors);
hooksecurefunc("CompactUnitFrame_UpdatePower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdatePower",UpdateAnchors);
hooksecurefunc("CompactUnitFrame_UpdatePowerColor",UpdatePowerColor);

do--	CastBar Hook
end

--------------------------
--[[	Options Setup	]]
--------------------------
local OptionsLayoutSection=1;--AddOn.GetPreferredOptionsLayoutSection();
AddOn.AddOptionsLayoutPadding(OptionsLayoutSection);

local OptionKeyToTextIndex={NamePlateText_ShowHealth=1,NamePlateText_ShowPower=2};
local function HealthPower_OnOptionsUpdate(_,key,val)
	local index,enabled=OptionKeyToTextIndex[key],(AddOn.Options.NamePlateText_ShowHealth or AddOn.Options.NamePlateText_ShowPower);
	if index then
		local control_abbreviate=AddOn.GetOptionsControl("NamePlateText_AbbreviateNumber");
		control_abbreviate:SetEnabled(enabled);
		control_abbreviate.text:SetFontObject(enabled and GameFontNormalSmall or GameFontDisableSmall);
	end

	for unitframe,text in pairs(TextObjects) do
		if index then text[index]:SetShown(val); end
		if enabled and unitframe.displayedUnit then
			UpdateAnchors(unitframe);
			if not index or key=="NamePlateText_ShowHealth" then
				UpdateHealth(unitframe);
			end
			if not index or key=="NamePlateText_ShowPower" then
				UpdatePowerColor(unitframe);
				UpdatePower(unitframe);
			end
		end
	end
end

AddOn.RegisterOptionsControl("NamePlateText_ShowHealth",true
	,"OnClick","SetChecked","GetChecked"
	,HealthPower_OnOptionsUpdate--	Callback
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24));
		button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_ShowHealth);
		return button;
	end
);

AddOn.RegisterOptionsControl("NamePlateText_ShowPower",true
	,"OnClick","SetChecked","GetChecked"
	,HealthPower_OnOptionsUpdate--	Callback
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24));
		button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_ShowPower);
		return button;
	end
);

AddOn.RegisterOptionsControl("NamePlateText_AbbreviateNumber",true
	,"OnClick","SetChecked","GetChecked"
	,HealthPower_OnOptionsUpdate--	Callback
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24,24,0));
		button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_AbbreviateNumber);
		return button;
	end
);

if not AddOn.IsVanillaClient then
	AddOn.AddOptionsLayoutPadding(OptionsLayoutSection);

	AddOn.RegisterOptionsControl("NamePlateText_ShowCastTime",true
		,"OnClick","SetChecked","GetChecked"
		,function(_,_,val)--	Callback
			for unitframe,text in pairs(TextObjects) do text[3]:SetShown(val); end
		end
		,function(panel)--	Initialization
			local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
			button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24));
			button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_ShowCastTime);
			return button;
		end
	);

	AddOn.RegisterOptionsControl("NamePlateText_ShowCastDiff",true
		,"OnClick","SetChecked","GetChecked"
		,function(_,_,val)--	Callback
			for unitframe,text in pairs(TextObjects) do text[4]:SetShown(val); end
		end
		,function(panel)--	Initialization
			local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
			button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24));
			button.text:SetText(AddOn.Localization.OptionsSetting_NamePlateText_ShowCastDiff);
			return button;
		end
	);
end
