--[[	PhantomPlates Personal NamePlate Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

--------------------------
--[[	Constants	]]
--------------------------
local DefaultYPos=-128;
local PowerRestState={
--	<not defined> = Rest at max
	[Enum.PowerType.Rage]=0;
	[Enum.PowerType.ComboPoints]=0;
	[Enum.PowerType.RunicPower]=0;
	[Enum.PowerType.SoulShards]=3;
	[Enum.PowerType.LunarPower]=0;
	[Enum.PowerType.HolyPower]=0;
	[Enum.PowerType.Maelstrom]=0;
	[Enum.PowerType.Chi]=0;
	[Enum.PowerType.Insanity]=0;
	[Enum.PowerType.ArcaneCharges]=0;
	[Enum.PowerType.Fury]=0;
	[Enum.PowerType.Pain]=0;
};

----------------------------------
--[[	CUF NamePlate Options	]]
----------------------------------
local PersonalNamePlateFrameOptions=setmetatable({
--	colorHealthBySelection=false;
	displayAggroHighlight=true;
	displayName=false;
	displayStatusText=true;
	healthText="none";

--[[	DefaultCompactNamePlatePlayerFrameOptions from Modern WoW
	displaySelectionHighlight=false;
--	displayAggroHighlight=false;
	displayName=false;
	fadeOutOfRange=false;
	colorNameBySelection=true;
	smoothHealthUpdates=false;
--	displayNameWhenSelected=false;
	showLevel=true;

	healthBarColorOverride=CreateColor(0,1,0);
	defaultBorderColor=CreateColor(0,0,0,1);
--]]
},{__index=DefaultCompactNamePlateFriendlyFrameOptions});

----------------------------------
--[[	Personal NamePlate	]]
----------------------------------
local PersonalNamePlate_Initialized=false;

local PersonalNamePlate=CreateFrame("Frame");
PersonalNamePlate:SetPoint("CENTER",0,DefaultYPos);
PersonalNamePlate:SetSize(128,32);
PersonalNamePlate:Hide();

PersonalNamePlate:RegisterEvent("PLAYER_LOGIN");

PersonalNamePlate:RegisterEvent("PLAYER_REGEN_ENABLED");
PersonalNamePlate:RegisterEvent("PLAYER_REGEN_DISABLED");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXHEALTH","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH_FREQUENT","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_SHOW","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_HIDE","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_DISPLAYPOWER","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXPOWER","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_UPDATE","player");

local function PersonalNamePlate_UpdateVisibility(show)
	if not PersonalNamePlate_Initialized then return; end--	Don't run if we haven't loaded yet (ex. Options callback via ADDON_LOADED fires before PLAYER_LOGIN)

--	Visibility Logic
	local showbypower=false; do
		local powertype,powertoken=UnitPowerType("player");
		showbypower=UnitPower("player",powertype)~=(PowerRestState[powertype] or UnitPowerMax("player",powertype));
	end

	show=(
			AddOn.Options.PersonalNamePlate_Enabled--	Enabled?
		and	UIParent:IsVisible()--	UI Shown?
		and	(	AddOn.Options.PersonalNamePlate_AlwaysShow--	Always Show?
			or	(	AddOn.OptionsPanel:IsVisible()--	Options Panel visible?
				or	(show and true)--	Force shown? (Cast to true; nil is already handled)
				or	(	show==nil--	Omitted arg? (Preserves false value from above)
					and	(	InCombatLockdown()--	Show in combat
						or	UnitHealth("player")<UnitHealthMax("player")--	Show if health missing
						or	(AddOn.Options.PowerBars_Enabled and showbypower)--	Show if power not at rest state (Only if PowerBars enabled)
					)
				)
			)
		)
	);

--	Apply if changed
	if show~=PersonalNamePlate:IsShown() then
		PersonalNamePlate:SetShown(show);
	end
end

PersonalNamePlate:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_LOGIN" then--	PersonalNamePlate Init (allow addons hooking NamePlateDriverFrame:OnNamePlateCreated() to work on this)
		NamePlateDriverFrame:OnNamePlateCreated(self);--	Creates UnitFrame and injects NamePlateBaseMixin
		if not AddOn.IsVanillaClient then--	Grab UnitFrame in TBC+ (Uses code from Modern client)
--			We want our UnitFrame, but asking the frame pool to create one for us spreads taint, so we hack the frame pool to create, but not store it
			self.UnitFrame=NamePlateDriverFrame.pools.pools.NamePlateUnitFrameTemplate:creationFunc();
			self.UnitFrame:SetParent(self);
			self.UnitFrame:SetPoint("TOPLEFT",self,"TOPLEFT");
			self.UnitFrame:EnableMouse(false);
			self:SetScript("OnSizeChanged",self.OnSizeChanged);
			self:OnSizeChanged();

			AddOn.OnNamePlateCreated(NamePlateDriverFrame,self)
--			NamePlateDriverFrame:AcquireUnitFrame(self);--	Original call for this (taints frame pool apparently)
		end
		self.UnitFrame.RaidTargetFrame:Hide();--		RaidTargetIcon is always hidden on player

--		Fix level text layering issue (Don't know why this bug doesn't happen on real NamePlates)
		self.UnitFrame.LevelFrame:SetFrameStrata(self.UnitFrame:GetFrameStrata());
		self.UnitFrame.LevelFrame:SetFrameLevel(self.UnitFrame.healthBar.border:GetFrameLevel()+1);

--		Layout Setup
--		self.UnitFrame.statusText:SetPoint("CENTER",self.UnitFrame.healthBar,"CENTER");

--		Apply Layout (NamePlateDriverFrame won't overwrite this since it never recieves NAME_PLATE_UNIT_ADDED for "player")
		CompactUnitFrame_SetUpFrame(PersonalNamePlate.UnitFrame,DefaultCompactNamePlateFriendlyFrameSetup);--	Expose creation as a friendly nameplate
		CompactUnitFrame_SetOptionTable(PersonalNamePlate.UnitFrame,PersonalNamePlateFrameOptions);--	Set our own options table
		CompactUnitFrame_UpdateAll(PersonalNamePlate.UnitFrame);--	Update display (Called by CompactUnitFrame_SetUpFrame(), but not CompactUnitFrame_SetOptionTable())

--[[		Setup Conversion Notes
--		These are in NamePlateDriverFrame:ApplyFrameOptions()
		self:OnOptionsUpdated();--	Defunct in classic
		NamePlateDriverFrame:UpdateInsetsForType("friendly",self);--	This isn't a real NamePlate

		NamePlateDriverFrame:OnUnitAuraUpdate("player");
		NamePlateDriverFrame:OnRaidTargetUpdate();
--]]

--		Safe to update visibility now
		PersonalNamePlate_Initialized=true;
	end

--	Force shown if PLAYER_REGEN_DISABLED is firing, otherwise use automated logic (InCombatLockdown() isn't true yet)
	PersonalNamePlate_UpdateVisibility(event=="PLAYER_REGEN_DISABLED" or nil);
end);

PersonalNamePlate:SetScript("OnShow",function(self)
	self:OnAdded("player",NamePlateDriverFrame);
	NamePlateDriverFrame:OnUnitAuraUpdate("player");
end);
PersonalNamePlate:SetScript("OnHide",function(self) return self:OnRemoved(); end);

local function OptionsPanel_OnVisibilityChanged() return PersonalNamePlate_UpdateVisibility(); end
AddOn.OptionsPanel:HookScript("OnShow",OptionsPanel_OnVisibilityChanged);
AddOn.OptionsPanel:HookScript("OnHide",OptionsPanel_OnVisibilityChanged);
UIParent:HookScript("OnShow",OptionsPanel_OnVisibilityChanged);
UIParent:HookScript("OnHide",OptionsPanel_OnVisibilityChanged);

--------------------------
--[[	Options Setup	]]
--------------------------
local OptionsLayoutSection=AddOn.GetPreferredOptionsLayoutSection();
AddOn.AddOptionsLayoutPadding(OptionsLayoutSection);

AddOn.RegisterOptionsControl("PersonalNamePlate_Enabled",true
	,"OnClick","SetChecked","GetChecked"
	,function(_,_,val)--	Callback
		local control_ypos=AddOn.GetOptionsControl("PersonalNamePlate_YPos");
		control_ypos:SetEnabled(val);
		do	local fontobj=val and GameFontHighlightSmall or GameFontDisableSmall;
			control_ypos.Text:SetFontObject(fontobj);
			control_ypos.Low:SetFontObject(fontobj);
			control_ypos.Mid:SetFontObject(fontobj);
			control_ypos.High:SetFontObject(fontobj);
		end

		local control_alwaysshow=AddOn.GetOptionsControl("PersonalNamePlate_AlwaysShow");
		control_alwaysshow:SetEnabled(val);
		control_alwaysshow.text:SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

		PersonalNamePlate_UpdateVisibility();
	end
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24));
		button.text:SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_Enabled);
		return button;
	end
);

AddOn.RegisterOptionsControl("PersonalNamePlate_AlwaysShow",false
	,"OnClick","SetChecked","GetChecked"
	,function() PersonalNamePlate_UpdateVisibility(); end--	Callback
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,24,24,0));
		button.text:SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_AlwaysShow);
		return button;
	end
);

AddOn.RegisterOptionsControl("PersonalNamePlate_YPos",DefaultYPos
	,"OnValueChanged","SetValue","GetValue"
	,function(self,_,val)--	Callback
		self.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,val);
		PersonalNamePlate:SetPoint("CENTER",0,val);
	end
	,function(panel)--	Initialization
		local slider=CreateFrame("Slider",nil,panel,"OptionsSliderTemplate");
		slider:SetPoint(AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,40,28,-16));
		slider:SetWidth(256);

		slider:SetMinMaxValues(-384,384);
		slider:SetValue(DefaultYPos);

		slider:SetValueStep(16);
		slider:SetStepsPerPage(1);
		slider:SetObeyStepOnDrag(true);

		slider.Mid=slider:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
		slider.Mid:SetPoint("TOP",slider,"BOTTOM",0,3);

		slider.Text:SetFontObject(GameFontHighlightSmall);
		slider.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,DefaultYPos);
		slider.Low:SetText(-384);
		slider.Mid:SetText(0);
		slider.High:SetText(384);

		OptionsSlider_YPos=slider;
		return slider;
	end
);

AddOn.HookOption("PowerBars_Enabled",function()
	if AddOn.Options.PersonalNamePlate_Enabled then PersonalNamePlate_UpdateVisibility(); end
end);
