--[[	PhantomPlates Core
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local Name,AddOn=...;
local Title=select(2,GetAddOnInfo(Name)):gsub("%s*v?[%d%.]+$","");
local Version=GetAddOnMetadata(Name,"Version");
local Author=GetAddOnMetadata(Name,"Author");

--------------------------
--[[	Options Tables	]]
--------------------------
local Defaults,Options={},nil;
AddOn.Options=Defaults;--	Modules can write to defaults while loading

--------------------------
--[[	Local Variables	]]
--------------------------
local OptionsCache={};
local OptionsControls={};
local OptionsCallbacks=setmetatable({},{__index=function(t,k)
	local dispatcher=AddOn.CreateDispatcher();
	t[k]=dispatcher; return dispatcher;
end});

--------------------------
--[[	Hook API	]]
--------------------------
AddOn.OnNamePlateCreated=AddOn.CreateDispatcher();
AddOn.OnNamePlateAdded=AddOn.CreateDispatcher();
AddOn.OnNamePlateRemoved=AddOn.CreateDispatcher();

hooksecurefunc(NamePlateDriverFrame,AddOn.IsVanillaClient and "OnNamePlateCreated" or "AcquireUnitFrame",AddOn.OnNamePlateCreated:EncloseDispatcher());
hooksecurefunc(NamePlateBaseMixin,"OnAdded",AddOn.OnNamePlateAdded:EncloseDispatcher());
hooksecurefunc(NamePlateBaseMixin,"OnRemoved",AddOn.OnNamePlateRemoved:EncloseDispatcher());


--[[
AddOn.DummyCreateNamePlateUnitFrame=nop;
--function AddOn.HookNamePlateCreated(func) return hooksecurefunc(NamePlateDriverFrame,"OnNamePlateCreated",func); end
function AddOn.HookNamePlateAdded(func) return hooksecurefunc(NamePlateBaseMixin,"OnAdded",func); end
function AddOn.HookNamePlateRemoved(func) return hooksecurefunc(NamePlateBaseMixin,"OnRemoved",func); end
function AddOn.HookNamePlateUFCreated(func)
	if AddOn.IsVanillaClient then
		return hooksecurefunc(NamePlateDriverFrame,AddOn.IsVanillaClient and "OnNamePlateCreated" or "AcquireUnitFrame",func);
	else
		hooksecurefunc(NamePlateDriverFrame,"AcquireUnitFrame",func);--	Default UI
		hooksecurefunc(AddOn,"DummyCreateNamePlateUnitFrame",func);--	Self Callback
	end
end
--]]

--------------------------
--[[	Options Panel	]]
--------------------------
local Panel=CreateFrame("Frame"); do
	Panel:Hide();

	Panel.name=Title;
	InterfaceOptions_AddCategory(Panel);--	Panel Registration
	AddOn.OptionsPanel=Panel;

	do	local title=Panel:CreateFontString(nil,"OVERLAY","GameFontNormalLarge");
		title:SetPoint("TOP",0,-12);
		title:SetText(Title);

		local author=Panel:CreateFontString(nil,"OVERLAY","GameFontNormalSmall");
		author:SetPoint("TOP",title,"BOTTOM",0,0);
		author:SetTextColor(1,0.5,0.25);
		author:SetFormattedText(AddOn.Localization.Options_ByAuthor_Format,Author);

		local ver=Panel:CreateFontString(nil,"OVERLAY","GameFontNormalSmall");
		ver:SetPoint("TOPLEFT",title,"TOPRIGHT",4,0);
		ver:SetTextColor(0.5,0.5,0.5);
		ver:SetText("v"..Version);
	end
end

Panel:RegisterEvent("ADDON_LOADED");
Panel:SetScript("OnEvent",function(self,event,...)
	if event=="ADDON_LOADED" and (...)==Name then
--		Load Options
		Options=PhantomPlates_Options;
		if not Options then Options={}; PhantomPlates_Options=Options; end
		AddOn.Options=Options;

--		Trim/Fill Options
--		for key,val in pairs(Options) do if Defaults[key]==nil then Options[key]=nil; end end
		for key,val in pairs(Defaults) do if Options[key]==nil then Options[key]=val; end end

--		Send updates to modules
		for key,control in pairs(OptionsControls) do
			OptionsCallbacks[key](control,key,Options[key]);
		end

--		Don't need this event anymore
		self:UnregisterEvent(event);
	end
end);

function Panel.okay()
	table.wipe(OptionsCache);--	Options already set by preview, just cleanup
end

function Panel.cancel()
	for key,val in pairs(OptionsCache) do
		if val~=Options[key] then
			Options[key]=val;--	Restore previous setting
			OptionsCallbacks[key](OptionsControls[key],key,val);
		end
	end
	table.wipe(OptionsCache);--	Cleanup
end

function Panel.refresh()
	table.wipe(OptionsCache);--	Sanity check
	for key,control in pairs(OptionsControls) do
		local val,setter=Options[key],control.Setter;
		OptionsCache[key]=val;--	Save current options for Panel.cancel()

		if type(setter)~="function" then setter=control[setter]; end
		setter(control,Options[key]);
	end
end

function Panel.default()
	local errhnd=geterrorhandler();
	for key,control in pairs(OptionsControls) do
		local val=Defaults[key];
		Options[key]=val;--	Restore default
		OptionsCallbacks[key](control,key,val);
	end
end

----------------------------------
--[[	Dynamic Options Layout	]]
----------------------------------
local DefaultLayoutPadding=24;
local LayoutSections={
	{
		Point="TOPLEFT";
		RelPoint="TOPLEFT";
		XOff=24;
		YOff=-48;
		VPos=0;
	};
	{
		Point="TOPLEFT";
		RelPoint="TOP";
		XOff=0;
		YOff=-48;
		VPos=0;
	};
};

function AddOn.GetPreferredOptionsLayoutSection()
	local lowpos,lowsection;
	for id,section in ipairs(LayoutSections) do
		local pos=section.VPos;
		if not lowpos or pos<lowpos then lowpos,lowsection=pos,id; end
	end
	return lowsection;
end

function AddOn.AddOptionsLayoutPadding(section,padding)
	local pos=LayoutSections[section].VPos; if pos>0 then
		LayoutSections[section].VPos=pos+(padding or DefaultLayoutPadding);
	end
end

function AddOn.GetNextOptionsLayoutSectionPosition(section,heightinc,xoffset,yoffset)
	local ptr=LayoutSections[section];
	local pos=ptr.VPos;

	if heightinc then ptr.VPos=pos+heightinc; end
	return ptr.Point,Panel,ptr.RelPoint,ptr.XOff+(xoffset or 0),ptr.YOff-pos+(yoffset or 0);
end

----------------------------------
--[[	Options Registration	]]
----------------------------------
local function OptionsControl_OnScriptCallback(self)
	local key,getter=self.OptionsKey,self.Getter;
	if type(getter)~="function" then getter=self[getter]; end
	local val=getter(self);

	Options[key]=val;
	OptionsCallbacks[key](self,key,val);
end

function AddOn.HookOption(key,func) OptionsCallbacks[key]:AddCallback(func); end

function AddOn.RegisterOptionsControl(key,default,script,setter,getter,callback,init)
	local errhnd=geterrorhandler();
	local ok,control=xpcall(init,errhnd,Panel);
	if not (ok and xpcall(assert,errhnd,control,"Init function returned nil")) then return; end

	control.OptionsKey,control.Setter,control.Getter=key,setter,getter;
	OptionsControls[key]=control;
	if callback then table.insert(OptionsCallbacks[key],1,callback); end--	We want the callback registered with the control to be the first to run

	control:HookScript(script,OptionsControl_OnScriptCallback);
	Defaults[key]=default;
	return control;
end

function AddOn.GetOptionsControl(key) return OptionsControls[key]; end
